@@ -890,9 +890,11 @@ sub onsel_list_editstorage {
 
             foreach my $snapid (@sids) {
                 next if ($snapid eq $id);
-                my $ISnapshot = IMachine_findSnapshot($IMachine, $snapid);
-                my $sname = ISnapshot_getName($ISnapshot);
-                $snames .= "$sname, ";
+                if (IMachine_getSnapshotCount($IMachine)) { # Just because the medium says its attached to a snapshot, that snapshot may not longer exist.
+                    my $ISnapshot = IMachine_findSnapshot($IMachine, $snapid);
+                    my $sname = ISnapshot_getName($ISnapshot) if ($ISnapshot);
+                    $snames .= "$sname, " if ($sname);
+                }
             }
 
             if ($snames) {
@@ -924,7 +926,7 @@ sub onsel_list_editstorage {
     sub getsel_list_vmmdvd { return \%selected; }
 
     # Fill the DVD media list in the VMM
-    sub fill_list_vmmdvd() {
+    sub fill_list_vmmdvd {
         &busy_pointer($gui{dialogVMM}, 1);
         &clr_list_vmm($gui{treestoreVMMDVD});
         my $IMediumRef = &get_all_media('DVD');
@@ -986,9 +988,11 @@ sub onsel_list_editstorage {
 
             foreach my $snapid (@sids) {
                 next if ($snapid eq $id);
-                my $ISnapshot = IMachine_findSnapshot($IMachine, $snapid);
-                my $sname = ISnapshot_getName($ISnapshot);
-                $snames .= "$sname, ";
+                if (IMachine_getSnapshotCount($IMachine)) { # Just because the medium says its attached to a snapshot, that snapshot may not longer exist.
+                    my $ISnapshot = IMachine_findSnapshot($IMachine, $snapid);
+                    my $sname = ISnapshot_getName($ISnapshot) if ($ISnapshot);
+                    $snames .= "$sname, " if ($sname);
+                }
             }
 
             if ($snames) {
@@ -1453,7 +1457,7 @@ sub onsel_list_shared {
             if (&bl(IMachine_getAccessible($machine))) {
                 $guestlist{$machine}{name} = IMachine_getName($machine);
                 my ($group) = IMachine_getGroups($machine); # We only care about the first group returned
-                $group =~ s/^\///; # Leading / is optional so always remove or simplicity
+                $group =~ s/^\///; # Leading / is optional so always remove for simplicity
                 my @components = split('/', $group);
                 my $piter = undef;
                 ($node, $piter) = &add_guest_group($node, $_, $piter) foreach (@components);
@@ -1462,7 +1466,17 @@ sub onsel_list_shared {
             else { $inaccessible = 1; }
         }
 
-        foreach my $machine (sort { lc($guestlist{$a}{name}) cmp lc($guestlist{$b}{name}) } (keys %guestlist)) {
+        # Lets sort the guest list according to preference
+        my @machinelist;
+
+        if ($prefs{AUTOSORTGUESTLIST}) {
+            foreach my $m (sort { lc($guestlist{$a}{name}) cmp lc($guestlist{$b}{name}) } (keys %guestlist)) {
+                push(@machinelist, $m);
+            }
+        }
+        else { @machinelist = sort(keys %guestlist) };
+
+        foreach my $machine (@machinelist) {
             my $ISnapshot = IMachine_getCurrentSnapshot($machine);
             my $osid = IMachine_getOSTypeId($machine);
             my $uuid = IMachine_getId($machine);
@@ -1508,4 +1522,96 @@ sub onsel_list_shared {
     }
 }
 
+# Block handling profiles
+{
+    my %selected = (Name => '');
+
+    # If pname exists we're calling this method directly, otherwise we're calling it from the button
+    sub addrow_profile {
+        my ($widget, $pname, $url, $username, $password) = @_;
+        my $iter = $gui{liststoreProfiles}->append;
+
+        if ($pname) {
+            $gui{liststoreProfiles}->set($iter, 0, $pname,
+                                                1, $url,
+                                                2, $username,
+                                                3, $password);
+        }
+        else {
+            $pname = 'Unnamed' . int(rand(9999));
+            $gui{liststoreProfiles}->set($iter, 0, $pname,
+                                            1, 'http://localhost:18083',
+                                            2, '',
+                                            3, '');
+            $gui{treeviewConnectionProfiles}->get_selection()->select_iter($iter);
+            &onsel_list_profile();
+        }
+    }
+
+    sub getsel_list_profile { return \%selected; }
+
+    sub onsel_list_profile {
+        my $model = $gui{treeviewConnectionProfiles}->get_model();
+        my $iter = $gui{treeviewConnectionProfiles}->get_selection->get_selected() ? $gui{treeviewConnectionProfiles}->get_selection->get_selected() : $model->get_iter_first();
+        my @row = $model->get($iter);
+        $selected{$_} = shift @row foreach ('Name', 'URL', 'Username', 'Password');
+        $gui{entryPrefsConnectionProfileName}->set_text($selected{Name});
+        $gui{entryPrefsConnectionProfileURL}->set_text($selected{URL});
+        $gui{entryPrefsConnectionProfileUsername}->set_text($selected{Username});
+        $gui{entryPrefsConnectionProfilePassword}->set_text($selected{Password});
+        $gui{buttonPrefsConnectionProfileDelete}->set_sensitive(1);
+        $gui{tablePrefsProfile}->set_sensitive(1);
+    }
+
+    # Delete a connection profile
+    sub remove_profile {
+        my ($widget) = @_;
+        my $model = $gui{treeviewConnectionProfiles}->get_model();
+        my $iter = $gui{treeviewConnectionProfiles}->get_selection->get_selected();
+        my $nextiter = $model->iter_next($iter);
+
+        if ($gui{liststoreProfiles}->iter_is_valid($iter)) {
+            $gui{entryPrefsConnectionProfileName}->set_text('');
+            $gui{entryPrefsConnectionProfileURL}->set_text('');
+            $gui{entryPrefsConnectionProfileUsername}->set_text('');
+            $gui{entryPrefsConnectionProfilePassword}->set_text('');
+            $gui{liststoreProfiles}->remove($iter);
+            $gui{buttonPrefsConnectionProfileDelete}->set_sensitive(0);
+            $gui{tablePrefsProfile}->set_sensitive(0);
+        }
+
+        if ($nextiter) {
+            $gui{treeviewConnectionProfiles}->get_selection()->select_iter($nextiter);
+            &onsel_list_profile();
+        }
+    }
+
+    sub profile_name_change {
+        my $model = $gui{treeviewConnectionProfiles}->get_model();
+        my $iter = $gui{treeviewConnectionProfiles}->get_selection->get_selected();
+        $gui{liststoreProfiles}->set_value($iter, 0, $gui{entryPrefsConnectionProfileName}->get_text()) if ($iter);
+    }
+
+    sub profile_url_change {
+        my $model = $gui{treeviewConnectionProfiles}->get_model();
+        my $iter = $gui{treeviewConnectionProfiles}->get_selection->get_selected();
+        $gui{liststoreProfiles}->set_value($iter, 1, $gui{entryPrefsConnectionProfileURL}->get_text()) if ($iter);
+    }
+
+        sub profile_username_change {
+        my $model = $gui{treeviewConnectionProfiles}->get_model();
+        my $iter = $gui{treeviewConnectionProfiles}->get_selection->get_selected();
+        $gui{liststoreProfiles}->set_value($iter, 2, $gui{entryPrefsConnectionProfileUsername}->get_text()) if ($iter);
+    }
+
+        sub profile_password_change {
+        my $model = $gui{treeviewConnectionProfiles}->get_model();
+        my $iter = $gui{treeviewConnectionProfiles}->get_selection->get_selected();
+        $gui{liststoreProfiles}->set_value($iter, 3, $gui{entryPrefsConnectionProfilePassword}->get_text()) if ($iter);
+    }
+
+}
+
+
+
 1;
