@@ -8,7 +8,7 @@ $endpoint = 'http://localhost:18083';
 $fault = sub{}; # Do nothing with faults until connected
 &rbprefs_get();
 &restore_window_pos('windowMain');
-&addrow_log("Welcome to $gui{appname} $gui{appver} (for use with VirtualBox 5.0.x)");
+&addrow_log("Welcome to $gui{appname} $gui{appver}");
 getopts("H:u:p:h", \%cmdopts);
 
 if($cmdopts{h}) {
@@ -57,11 +57,6 @@ sub virtualbox_logon {
         $gui{servermem} = Glib::Timeout->add(300000, \&update_server_membar);
         return 1;
     }
-    # SHOULD THIS BE DELETED????
-    #else {
-    #    #&show_err_msg('connect', " ($url)");
-    #    return 0;
-    #}
 }
 
 # Logoff and perform various necessary cleanups
@@ -91,46 +86,41 @@ sub virtualbox_logoff {
     &clr_vhost(); # Clear vhost so values are not retain on subsequent connections
 }
 
+# Fill in the fields when a profile is selected
+sub select_profile {
+    my ($widget) = @_;
+    # For some reason this fires when deleting a profile from the profile manage.
+    my $model = $widget->get_model();
+    my $iter = $widget->get_active_iter();
+    my @row = $model->get($iter) if ($iter and $model);
+
+    if (@row) {
+        $gui{entryConnectURL}->set_text($row[1]);
+        $gui{entryConnectUser}->set_text($row[2]);
+        $gui{entryConnectPassword}->set_text($row[3]);
+    }
+}
+
 # Be careful with use of $vhost in this sub!
 sub show_dialog_connect {
     my ($widget) = @_;
     my $response = 'cancel';
 
-    $gui{liststoreConnectURL}->clear();
-    $gui{liststoreConnectUser}->clear();
-    $gui{checkbuttonConnectSave}->set_active($prefs{SAVEURLUSER});
     $gui{checkbuttonConnectSSL}->set_active($prefs{SSLVERIFY});
 
-    foreach (keys (%{$prefs{URL}})) {
-        my $iter = $gui{liststoreConnectURL}->append();
-        $gui{liststoreConnectURL}->set($iter, 0, $_);
-    }
-
-    foreach (keys (%{$prefs{USER}})) {
-        my $iter = $gui{liststoreConnectUser}->append();
-        $gui{liststoreConnectUser}->set($iter, 0, $_);
-    }
-
     # Check if a command line automatic login has been requested. We do this here
     # to try an ensure that a cmdline login is as close as possible to a GUI login
     if ($widget eq 'AUTO') {
-        my $iter = $gui{liststoreConnectURL}->append();
-        $gui{liststoreConnectURL}->set($iter, 0, $cmdopts{H});
-        $gui{comboboxentryConnectURL}->set_active_iter($iter);
-
-        if ($cmdopts{u}) {
-            $iter = $gui{liststoreConnectUser}->append();
-            $gui{liststoreConnectUser}->set($iter, 0, $cmdopts{u});
-            $gui{comboboxentryConnectUser}->set_active_iter($iter);
-        }
-
-        if ($cmdopts{p}) { $gui{entryConnectPassword}->set_text($cmdopts{p}); }
+        $gui{entryConnectURL}->set_text($cmdopts{H});
+        $gui{entryConnectUser}->set_text($cmdopts{u}) if ($cmdopts{u});
+        $gui{entryConnectPassword}->set_text($cmdopts{p}) if ($cmdopts{p});
 
         $response = 'ok';
         &addrow_log("Attempting automatic login to $endpoint");
     }
     else {
         # Normal GUI login
+        $gui{comboboxConnectProfile}->set_active(-1); # We don't want a current active
         $response = $gui{dialogConnect}->run;
         $gui{dialogConnect}->hide;
     }
@@ -138,18 +128,15 @@ sub show_dialog_connect {
     $gui{dialogConnect}->get_display->flush;
 
     if ($response eq 'ok') {
-        my $url = $gui{comboboxentryConnectURL}->get_active_text();
-        my $user = $gui{comboboxentryConnectUser}->get_active_text();
+        my $url = $gui{entryConnectURL}->get_text();
+        my $user = $gui{entryConnectUser}->get_text();
         $url = $endpoint if (!$url);
         $url =~ s/(\/)+$//g; # Remove all trailing
         $url = "http://$url" if ($url !~ m/^.+:\/\//);
         $url = "$url:18083" if ($url !~ m/:\d+$/);
-
-        if ($gui{checkbuttonConnectSave}->get_active()) {
-            $prefs{URL}{$url} = 'URL' if ($url);
-            $prefs{USER}{$user} = 'USER' if ($user);
-            &rbprefs_save();
-        }
+        $prefs{URL}{$url} = 'URL' if ($url);
+        $prefs{USER}{$user} = 'USER' if ($user);
+        &rbprefs_save();
 
         # If we got a successful logon
         if (&virtualbox_logon($url, $user, $gui{entryConnectPassword}->get_text())) {
@@ -178,6 +165,7 @@ sub show_dialog_connect {
     }
 }
 
+# Shows the about dialog
 sub show_dialog_about {
     $gui{aboutdialog}->run;
     $gui{aboutdialog}->hide;
@@ -304,7 +292,7 @@ sub show_dialog_snapshotdetails {
             my $iter_e = $gui{textbufferSnapshotDescription}->get_end_iter();
             ISnapshot_setDescription($$snapref{ISnapshot}, $gui{textbufferSnapshotDescription}->get_text($iter_s, $iter_e, 0));
             ISnapshot_setName($$snapref{ISnapshot}, $gui{entrySnapshotName}->get_text());
-            &fill_list_snapshots();
+            &fill_list_guest();
         }
     }
 }
@@ -597,7 +585,7 @@ sub open_remote_display {
 
         if ($$IVRDEServerInfo{port} > 0) {
             my $rdpcmd = $prefs{RDPCLIENT};
-            my ($user, $pass) = ($gui{comboboxentryConnectUser}->get_active_text(), $gui{entryConnectPassword}->get_text());
+            my ($user, $pass) = ($gui{entryConnectUser}->get_text(), $gui{entryConnectPassword}->get_text());
             my $dst = $endpoint;
             $dst =~ s/^.*:\/\///;
             $dst =~ s/:\d+$//;
@@ -1504,7 +1492,7 @@ sub update_server_membar {
         my $memsize = IHost_getMemorySize($IHost);
         my $memavail = IHost_getMemoryAvailable($IHost);
         my $memused = ($memsize - $memavail);
-        $gui{progressbarMem}->set_text("$memused MB / $memavail MB / $memsize MB (Server Memory: Used/Free/Total)");
+        $gui{progressbarMem}->set_text("Memory ($memsize MB): $memused MB Used / $memavail MB Free");
         $gui{progressbarMem}->set_fraction(($memused / $memsize));
     }
     return 1; # Return 1 to stop the timer from being removed
@@ -1558,4 +1546,32 @@ sub bytesToX {
     return $val;
 }
 
+# Simple XOR with password and key
+sub xor_pass {
+    my ($pass, $key) = @_;
+
+    my $encpass = '';
+    foreach my $char (split //, $pass) {
+        my $decode = chop($key);
+        $encpass .= chr(ord($char) ^ ord($decode));
+        $key = $decode . $key;
+    }
+    return $encpass;
+}
+
+# Returns a random string of printable ASCII characters up to the requested length
+sub random_key {
+    my ($length) = @_;
+    return '' if ($length < 1);
+    my @letters = ('a'..'z', 'A'..'Z', '0'..'9');
+    my $string = '';
+    foreach (1..$length) { $string .= $letters[rand(62)]; }
+    return $string;
+}
+
+sub set_connection_prof_inactive() {
+    $gui{comboboxConnectProfile}->set_active(-1);
+}
+
+
 1;
