@@ -2,36 +2,38 @@
 use strict;
 use warnings;
 our %gui;
-our %prefs = (RDPCLIENT       => 'xfreerdp -g %Xx%Y --plugin cliprdr --plugin rdpsnd -T "%n - RemoteBox" %h:%p',
-              HEARTBEAT       => 1,
-              ADDADDITIONS    => 1,
-              RDPAUTOOPEN     => 1,
-              MSGLOGEXPAND    => 1,
-              GUESTLISTEXPAND => 1,
-              EXTENDEDDETAILS => 0,
-              SSLVERIFY       => 0,
-              DEFRDPPORTS     => '3389-4389',
-              AUTOHINTDISP    => 0,
-              AUTOHINTDISPX   => 800,
-              AUTOHINTDISPY   => 600,
-              AUTOHINTDISPD   => 32,
-              STOPTYPE        => 'INSTANT',
-              EXPANDDETGEN    => 1,
-              EXPANDDETSYS    => 1,
-              EXPANDDETDISP   => 1,
-              EXPANDDETSTOR   => 1,
-              EXPANDDETAUDIO  => 1,
-              EXPANDDETNET    => 1,
-              EXPANDDETIO     => 1,
-              EXPANDDETUSB    => 1,
-              EXPANDDETSHARE  => 1,
-              EXPANDDETRUN    => 1,
-              EXPANDDETDESC   => 1);
+our %prefs = (RDPCLIENT         => 'xfreerdp /size:%Xx%Y /bpp:32 +clipboard /sound /t:"%n - RemoteBox" /v:%h:%p',
+              HEARTBEAT         => 1,
+              ADDADDITIONS      => 1,
+              RDPAUTOOPEN       => 1,
+              MSGLOGEXPAND      => 1,
+              GUESTLISTEXPAND   => 1,
+              AUTOSORTGUESTLIST => 1,
+              EXTENDEDDETAILS   => 0,
+              SSLVERIFY         => 0,
+              DEFRDPPORTS       => '3389-4389',
+              AUTOHINTDISP      => 0,
+              AUTOHINTDISPX     => 1280,
+              AUTOHINTDISPY     => 1024,
+              AUTOHINTDISPD     => 32,
+              STOPTYPE          => 'INSTANT',
+              EXPANDDETGEN      => 1,
+              EXPANDDETSYS      => 1,
+              EXPANDDETDISP     => 1,
+              EXPANDDETSTOR     => 1,
+              EXPANDDETAUDIO    => 1,
+              EXPANDDETNET      => 1,
+              EXPANDDETIO       => 1,
+              EXPANDDETUSB      => 1,
+              EXPANDDETSHARE    => 1,
+              EXPANDDETRUN      => 1,
+              EXPANDDETDESC     => 1);
 
 # Import the current remotebox preferences
 sub rbprefs_get {
     $gui{PREFSDIR} = Glib::get_user_config_dir();
     $gui{CONFIGFILE} = "$gui{PREFSDIR}/remotebox.conf";
+    $gui{PROFILESFILE} = "$gui{PREFSDIR}/remotebox-profiles.conf";
     $gui{THUMBDIR} = "$gui{PREFSDIR}/remoteboxthumbs";
 
     # See if we can make some more useful defaults for OSs which don't have xfreerdp
@@ -53,11 +55,25 @@ sub rbprefs_get {
 
     $gui{expanderMessages}->set_expanded($prefs{MSGLOGEXPAND});
     $gui{checkbuttonShowDetails}->set_active($prefs{EXTENDEDDETAILS});
+
+    # Load in the connection profiles
+    if (open(PROFILES, '<', $gui{PROFILESFILE})) {
+        my @contents = <PROFILES>;
+        chomp(@contents);
+        close(PROFILES);
+        foreach (sort(@contents)) {
+            my ($pname, $url, $username, $password, $key) = split("\a", $_);
+            my @pass = split(' ', $password);
+            $password = pack("C*", @pass);
+            $password = &xor_pass($password, $key);
+            &addrow_profile(undef, $pname, $url, $username, $password);
+        }
+    }
 }
 
 # Save the current remotebox preferences
 sub rbprefs_save {
-	mkdir($gui{PREFSDIR}, 0755) unless (-e $gui{PREFSDIR});
+    mkdir($gui{PREFSDIR}, 0755) unless (-e $gui{PREFSDIR});
 
     if (open(PREFS, '>', $gui{CONFIGFILE})) {
         foreach my $key (keys %prefs) {
@@ -67,11 +83,43 @@ sub rbprefs_save {
             elsif ($key eq 'USER') {
                 foreach (keys %{$prefs{USER}}) { print PREFS "USER=$_\n"; }
             }
-            else { print PREFS "$key=$prefs{$key}\n"; }
+            else { print(PREFS "$key=$prefs{$key}\n"); }
         }
         close(PREFS);
     }
     else { warn "Unable to save preferences: $gui{CONFIGFILE}\n"; }
+
+
+}
+
+# Saves the profiles.
+# DO NOT call this from rbprefs_save otherwise there will be a race condition which will
+# empty the profiles file.
+sub rbprofiles_save {
+    # As this file might contain passwords, we'll create it 0600, but only if it's the first time we're creating it.
+    my $new = 1;
+    $new = 0 if (-e $gui{PROFILESFILE});
+
+    # Save the connection profiles
+    if (open(PROFILES, '>', $gui{PROFILESFILE})) {
+        chmod(0600, $gui{PROFILESFILE}) if ($new);
+        my $iter = $gui{liststoreProfiles}->get_iter_first();
+
+        while($iter) {
+            print(PROFILES $gui{liststoreProfiles}->get_value($iter, 0) . "\a");
+            print(PROFILES $gui{liststoreProfiles}->get_value($iter, 1) . "\a");
+            print(PROFILES $gui{liststoreProfiles}->get_value($iter, 2) . "\a");
+            my $password = $gui{liststoreProfiles}->get_value($iter, 3);
+            my $key = &random_key(length($password));
+            $password = &xor_pass($password, $key);
+            my @ordpass = unpack("C*", $password);
+            $password = join(' ', @ordpass);
+            print(PROFILES "$password\a");
+            print(PROFILES "$key\n");
+            $iter = $gui{liststoreProfiles}->iter_next($iter);
+        }
+        close(PROFILES);
+    }
 }
 
 # Save RemoteBox's window position for restoration later
@@ -199,6 +247,7 @@ sub show_dialog_prefs {
     $gui{checkbuttonPrefsRDPAuto}->set_active($prefs{RDPAUTOOPEN});
     $gui{checkbuttonPrefsHeartbeat}->set_active($prefs{HEARTBEAT});
     $gui{checkbuttonPrefsAddAdditions}->set_active($prefs{ADDADDITIONS});
+    $gui{checkbuttonPrefsAutoSortGuestList}->set_active($prefs{AUTOSORTGUESTLIST});
     $gui{checkbuttonPrefsAutoHint}->set_active($prefs{AUTOHINTDISP});
     $gui{spinbuttonPrefsWidth}->set_value($prefs{AUTOHINTDISPX});
     $gui{spinbuttonPrefsHeight}->set_value($prefs{AUTOHINTDISPY});
@@ -220,6 +269,7 @@ sub show_dialog_prefs {
         $prefs{RDPAUTOOPEN} = int($gui{checkbuttonPrefsRDPAuto}->get_active());
         $prefs{HEARTBEAT} = int($gui{checkbuttonPrefsHeartbeat}->get_active());
         $prefs{ADDADDITIONS} = int($gui{checkbuttonPrefsAddAdditions}->get_active());
+        $prefs{AUTOSORTGUESTLIST} = int($gui{checkbuttonPrefsAutoSortGuestList}->get_active());
         $prefs{STOPTYPE} = &getsel_combo($gui{comboboxPrefsDefaultStop}, 1);
         $prefs{AUTOHINTDISP} = int($gui{checkbuttonPrefsAutoHint}->get_active());
         $prefs{AUTOHINTDISPX} = int($gui{spinbuttonPrefsWidth}->get_value());
@@ -229,19 +279,6 @@ sub show_dialog_prefs {
     }
 }
 
-# Clears the connection history so the connect dialog will be empty
-sub show_dialog_clearhistory {
-    my $response = $gui{messagedialogClearHistory}->run;
-    $gui{messagedialogClearHistory}->hide;
-
-    if ($response eq 'ok') {
-        $prefs{USER} = ();
-        $prefs{URL} = ();
-        &rbprefs_save();
-        &addrow_log('Cleared connection history.');
-    }
-}
-
 # Shows the VirtualBox preferences dialog
 sub show_dialog_vbprefs {
     my $vhost = &vhost();
@@ -267,6 +304,14 @@ sub show_dialog_vbprefs {
     &clr_vhost(); # VB Prefs changes can potentially alter vhost values, so clear them to be repopulated
 }
 
+# Shows the dialog for managing connection profiles
+sub show_dialog_profiles {
+    $gui{dialogProfiles}->run;
+    $gui{dialogProfiles}->hide;
+    &rbprofiles_save();
+}
+
+
 sub show_dialog_edithon {
     my $ifref = &getsel_list_vbprefshon();
     my @DHCPServers = IVirtualBox_getDHCPServers($gui{websn});
@@ -388,11 +433,6 @@ sub rbprefs_msglog {
     &rbprefs_save();
 }
 
-sub rbprefs_saveurluser {
-    $prefs{SAVEURLUSER} = $gui{checkbuttonConnectSave}->get_active();
-    &rbprefs_save();
-}
-
 sub rbprefs_sslverify {
     my $active = $gui{checkbuttonConnectSSL}->get_active();
     $ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = $active;
@@ -428,10 +468,12 @@ sub show_rdppreset_menu {
 sub set_rdppreset {
     my ($widget) = @_;
 
-    if ($widget eq $gui{menuitemrdppreset1}) { $gui{entryPrefsRDPClient}->set_text('xfreerdp /size:%Xx%Y /bpp:32 +clipboard /sound /multimedia /t:"%n - RemoteBox" /v:%h:%p'); }
+    if ($widget eq $gui{menuitemrdppreset1}) { $gui{entryPrefsRDPClient}->set_text('xfreerdp /size:%Xx%Y /bpp:32 +clipboard /sound /t:"%n - RemoteBox" /v:%h:%p'); }
     elsif ($widget eq $gui{menuitemrdppreset2}) { $gui{entryPrefsRDPClient}->set_text('xfreerdp -g %Xx%Y --plugin cliprdr --plugin rdpsnd -T "%n - RemoteBox" %h:%p'); }
     elsif ($widget eq $gui{menuitemrdppreset3}) { $gui{entryPrefsRDPClient}->set_text('rdesktop -r sound:local -r clipboard:PRIMARYCLIPBOARD -T "%n - RemoteBox" %h:%p'); }
     elsif ($widget eq $gui{menuitemrdppreset4}) { $gui{entryPrefsRDPClient}->set_text('mstsc /w:%X /h:%Y /v:%h:%p >nul'); }
 }
 
+
+
 1;
